@@ -2,24 +2,42 @@
 Contains all CMR query types.
 """
 
-try:
-    from urllib.parse import quote
-except ImportError:
-    from urllib import pathname2url as quote
-
-from datetime import datetime, timezone
+from abc import abstractmethod
+from datetime import date, datetime, timezone
 from inspect import getmembers, ismethod
 from re import search
+from typing_extensions import (
+    Any,
+    List,
+    Literal,
+    MutableMapping,
+    Optional,
+    Sequence,
+    Self,
+    Set,
+    SupportsFloat,
+    Tuple,
+    TypeAlias,
+    Union,
+    override,
+)
+from urllib.parse import quote
 
-from requests import get, exceptions
+import requests
 from dateutil.parser import parse as dateutil_parse
 
 CMR_OPS = "https://cmr.earthdata.nasa.gov/search/"
 CMR_UAT = "https://cmr.uat.earthdata.nasa.gov/search/"
 CMR_SIT = "https://cmr.sit.earthdata.nasa.gov/search/"
 
+DateLike: TypeAlias = Union[str, date, datetime]
+DayNightFlag: TypeAlias = Union[
+    Literal["day"], Literal["night"], Literal["unspecified"]
+]
+FloatLike: TypeAlias = Union[str, SupportsFloat]
+PointLike: TypeAlias = Tuple[FloatLike, FloatLike]
 
-class Query(object):
+class Query:
     """
     Base class for all CMR queries.
     """
@@ -32,15 +50,15 @@ class Query(object):
         "csv", "atom", "kml", "native"
     ]
 
-    def __init__(self, route, mode=CMR_OPS):
-        self.params = {}
-        self.options = {}
+    def __init__(self, route: str, mode: str = CMR_OPS):
+        self.params: MutableMapping[str, Any] = {}
+        self.options: MutableMapping[str, Any] = {}
         self._route = route
         self.mode(mode)
-        self.concept_id_chars = []
-        self.headers = None
+        self.concept_id_chars: Set[str] = set()
+        self.headers: MutableMapping[str, str] = {}
 
-    def get(self, limit=2000):
+    def get(self, limit: int = 2000) -> Sequence[Any]:
         """
         Get all results up to some limit, even if spanning multiple pages.
 
@@ -48,42 +66,41 @@ class Query(object):
         :returns: query results as a list
         """
 
-        page_size = min(limit, 2000)
         url = self._build_url()
 
         results = []
+        headers = dict(self.headers or {})
         more_results = True
-        while more_results == True:
+        n_results = 0
 
-            # Only get what we need
-            page_size = min(limit - len(results), page_size)
-            response = get(url, headers=self.headers, params={'page_size': page_size})
-            if self.headers == None:
-                self.headers = {}
-            self.headers['cmr-search-after'] = response.headers.get('cmr-search-after')
+        while more_results:
+            # Only get what we need on the last page.
+            page_size = min(limit - n_results, 2000)
+            response = requests.get(
+                url, headers=headers, params={"page_size": page_size}
+            )
+            response.raise_for_status()
 
-            try:
-                response.raise_for_status()
-            except exceptions.HTTPError as ex:
-                raise RuntimeError(ex.response.text)
+            # Explicitly track the number of results we have because the length
+            # of the results list will only match the number of entries fetched
+            # when the format is JSON.  Otherwise, the length of the results
+            # list is the number of *pages* fetched, not the number of *items*.
+            n_results += page_size
+
+            results.extend(
+                response.json()["feed"]["entry"]
+                if self._format == "json"
+                else [response.text]
+            )
+
+            if cmr_search_after := response.headers.get("cmr-search-after"):
+                headers["cmr-search-after"] = cmr_search_after
+
+            more_results = n_results < limit and cmr_search_after is not None
 
-            if self._format == "json":
-                latest = response.json()['feed']['entry']
-            else:
-                latest = [response.text]
-                
-            results.extend(latest)
-            
-            if page_size > len(response.json()['feed']['entry']) or len(results) >= limit:
-                more_results = False
-                
-        # This header is transient. We need to get rid of it before we do another different query
-        if self.headers['cmr-search-after']:
-            del self.headers['cmr-search-after']
-            
         return results
 
-    def hits(self):
+    def hits(self) -> int:
         """
         Returns the number of hits the current query will return. This is done by
         making a lightweight query to CMR and inspecting the returned headers.
@@ -93,16 +110,12 @@ class Query(object):
 
         url = self._build_url()
 
-        response = get(url, headers=self.headers, params={'page_size': 0})
-
-        try:
-            response.raise_for_status()
-        except exceptions.HTTPError as ex:
-            raise RuntimeError(ex.response.text)
+        response = requests.get(url, headers=self.headers, params={"page_size": 0})
+        response.raise_for_status()
 
         return int(response.headers["CMR-Hits"])
 
-    def get_all(self):
+    def get_all(self) -> Sequence[Any]:
         """
         Returns all of the results for the query. This will call hits() first to determine how many
         results their are, and then calls get() with that number. This method could take quite
@@ -113,26 +126,22 @@ class Query(object):
 
         return self.get(self.hits())
 
-    def parameters(self, **kwargs):
+    def parameters(self, **kwargs: Any) -> Self:
         """
         Provide query parameters as keyword arguments. The keyword needs to match the name
         of the method, and the value should either be the value or a tuple of values.
 
         Example: parameters(short_name="AST_L1T", point=(42.5, -101.25))
 
-        :returns: Query instance
+        :returns: self
         """
 
-        # build a dictionary of method names and their reference
-        methods = {}
-        for name, func in getmembers(self, predicate=ismethod):
-            methods[name] = func
+        methods = dict(getmembers(self, predicate=ismethod))
 
         for key, val in kwargs.items():
-
             # verify the key matches one of our methods
             if key not in methods:
-                raise ValueError("Unknown key {}".format(key))
+                raise ValueError(f"Unknown key {key}")
 
             # call the method
             if isinstance(val, tuple):
@@ -142,12 +151,12 @@ class Query(object):
 
         return self
 
-    def format(self, output_format="json"):
+    def format(self, output_format: str = "json") -> Self:
         """
         Sets the format for the returned results.
 
         :param output_format: Preferred output format
-        :returns: Query instance
+        :returns: self
         """
 
         if not output_format:
@@ -160,9 +169,9 @@ class Query(object):
                 return self
 
         # if we got here, we didn't find a matching format
-        raise ValueError("Unsupported format '{}'".format(output_format))
+        raise ValueError(f"Unsupported format: '{output_format}'")
 
-    def _build_url(self):
+    def _build_url(self) -> str:
         """
         Builds the URL that will be used to query CMR.
 
@@ -181,43 +190,32 @@ class Query(object):
             # list params require slightly different formatting
             if isinstance(val, list):
                 for list_val in val:
-                    formatted_params.append("{}[]={}".format(key, list_val))
+                    formatted_params.append(f"{key}[]={list_val}")
             elif isinstance(val, bool):
-                formatted_params.append("{}={}".format(key, str(val).lower()))
+                formatted_params.append(f"{key}={str(val).lower()}")
             else:
-                formatted_params.append("{}={}".format(key, val))
+                formatted_params.append(f"{key}={val}")
 
         params_as_string = "&".join(formatted_params)
 
         # encode options
-        formatted_options = []
+        formatted_options: List[str] = []
         for param_key in self.options:
             for option_key, val in self.options[param_key].items():
 
                 # all CMR options must be booleans
                 if not isinstance(val, bool):
-                    raise ValueError("parameter '{}' with option '{}' must be a boolean".format(
-                        param_key,
-                        option_key
-                    ))
+                    raise TypeError(
+                        f"parameter '{param_key}' with option '{option_key}' must be a boolean"
+                    )
 
-                formatted_options.append("options[{}][{}]={}".format(
-                    param_key,
-                    option_key,
-                    str(val).lower()
-                ))
+                formatted_options.append(f"options[{param_key}][{option_key}]={str(val).lower()}")
 
         options_as_string = "&".join(formatted_options)
-        res = "{}.{}?{}&{}".format(
-            self._base_url,
-            self._format,
-            params_as_string,
-            options_as_string
-        )
-        res = res.rstrip('&')
-        return res
+        res = f"{self._base_url}.{self._format}?{params_as_string}&{options_as_string}"
+        return res.rstrip('&')
 
-    def concept_id(self, IDs):
+    def concept_id(self, IDs: Union[str, Sequence[str]]) -> Self:
         """
         Filter by concept ID (ex: C1299783579-LPDAAC_ECS or G1327299284-LPDAAC_ECS, T12345678-LPDAAC_ECS, S12345678-LPDAAC_ECS)
 
@@ -228,7 +226,7 @@ class Query(object):
         If providing a service's concept ID here, it will uniquely identify those services.
 
         :param IDs: concept ID(s) to search by. Can be provided as a string or list of strings.
-        :returns: Query instance
+        :returns: self
         """
 
         if isinstance(IDs, str):
@@ -238,18 +236,19 @@ class Query(object):
         for ID in IDs:
             if ID.strip()[0] not in self.concept_id_chars:
                 raise ValueError(
-                    "Only concept ids that begin with '{}' can be provided: {}".format(self.concept_id_chars, ID))
+                    f"Only concept IDs that begin with '{self.concept_id_chars}' can be provided: {ID}"
+                )
 
         self.params["concept_id"] = IDs
 
         return self
 
-    def provider(self, provider):
+    def provider(self, provider: str) -> Self:
         """
         Filter by provider.
 
         :param provider: provider of tool.
-        :returns: Query instance
+        :returns: self
         """
 
         if not provider:
@@ -258,7 +257,8 @@ class Query(object):
         self.params['provider'] = provider
         return self
 
-    def _valid_state(self):
+    @abstractmethod
+    def _valid_state(self) -> bool:
         """
         Determines if the Query is in a valid state based on the parameters and options
         that have been set. This should be implemented by the subclasses.
@@ -268,25 +268,27 @@ class Query(object):
 
         raise NotImplementedError()
 
-    def mode(self, mode=CMR_OPS):
+    def mode(self, mode: str = CMR_OPS) -> Self:
         """
         Sets the mode of the api target to the given URL
         CMR_OPS, CMR_UAT, CMR_SIT are provided as class variables
 
         :param mode: Mode to set the query to target
-        :throws: Will throw if provided None
+        :returns: self
+        :raises: Will raise if provided None
         """
         if mode is None:
             raise ValueError("Please provide a valid mode (CMR_OPS, CMR_UAT, CMR_SIT)")
 
-        self._base_url = str(mode) + self._route
+        self._base_url = mode + self._route
+        return self
 
-    def token(self, token):
+    def token(self, token: str) -> Self:
         """
         Add token into authorization headers.
 
         :param token: Token from EDL Echo-Token or NASA Launchpad token.
-        :returns: Query instance
+        :returns: self
         """
 
         if not token:
@@ -296,18 +298,18 @@ class Query(object):
 
         return self
 
-    def bearer_token(self, bearer_token):
+    def bearer_token(self, bearer_token: str) -> Self:
         """
         Add token into authorization headers.
 
         :param token: Token from EDL token.
-        :returns: Query instance
+        :returns: self
         """
 
         if not bearer_token:
             return self
 
-        self.headers = {'Authorization': 'Bearer ' + bearer_token}
+        self.headers = {'Authorization': f'Bearer {bearer_token}'}
 
         return self
 
@@ -317,13 +319,13 @@ class GranuleCollectionBaseQuery(Query):
     Base class for Granule and Collection CMR queries.
     """
 
-    def online_only(self, online_only=True):
+    def online_only(self, online_only: bool = True) -> Self:
         """
         Only match granules that are listed online and not available for download.
         The opposite of this method is downloadable().
 
         :param online_only: True to require granules only be online
-        :returns: Query instance
+        :returns: self
         """
 
         if not isinstance(online_only, bool):
@@ -337,7 +339,12 @@ class GranuleCollectionBaseQuery(Query):
 
         return self
 
-    def temporal(self, date_from, date_to, exclude_boundary=False):
+    def temporal(
+        self,
+        date_from: Optional[DateLike],
+        date_to: Optional[DateLike],
+        exclude_boundary: bool = False,
+    ) -> Self:
         """
         Filter by an open or closed date range.
 
@@ -353,7 +360,7 @@ class GranuleCollectionBaseQuery(Query):
         iso_8601 = "%Y-%m-%dT%H:%M:%SZ"
 
         # process each date into a datetime object
-        def convert_to_string(date, default):
+        def convert_to_string(date: Optional[DateLike], default: datetime) -> str:
             """
             Returns the argument as an ISO 8601 or empty string.
             """
@@ -370,15 +377,16 @@ class GranuleCollectionBaseQuery(Query):
                 try:
                     date = datetime.combine(date, default.time())
                 except TypeError:
-                    msg = f"Date must be a date object or ISO 8601 string, not {date.__class__.__name__}."
-                    raise TypeError(msg)
+                    raise TypeError(
+                        f"Date must be a date object or ISO 8601 string, not {date.__class__.__name__}."
+                    ) from None
                 date = date.replace(tzinfo=timezone.utc)
             else:
                 # pass aware datetime and handle naive datetime by assuming utc
                 date = date if date.tzinfo else date.replace(tzinfo=timezone.utc)
-            
+
             # convert aware datetime to utc datetime
-            date = date.astimezone(timezone.utc)            
+            date = date.astimezone(timezone.utc)
 
             return date.strftime(iso_8601)
 
@@ -386,15 +394,14 @@ class GranuleCollectionBaseQuery(Query):
         date_to = convert_to_string(date_to, datetime(1, 12, 31, 23, 59, 59, tzinfo=timezone.utc))
 
         # if we have both dates, make sure from isn't later than to
-        if date_from and date_to:
-            if date_from > date_to:
-                raise ValueError("date_from must be earlier than date_to.")
+        if date_from and date_to and date_from > date_to:
+            raise ValueError("date_from must be earlier than date_to.")
 
         # good to go, make sure we have a param list
         if "temporal" not in self.params:
             self.params["temporal"] = []
 
-        self.params["temporal"].append("{},{}".format(date_from, date_to))
+        self.params["temporal"].append(f"{date_from},{date_to}")
 
         if exclude_boundary:
             self.options["temporal"] = {
@@ -403,12 +410,12 @@ class GranuleCollectionBaseQuery(Query):
 
         return self
 
-    def short_name(self, short_name):
+    def short_name(self, short_name: str) -> Self:
         """
         Filter by short name (aka product or collection name).
 
         :param short_name: name of collection
-        :returns: Query instance
+        :returns: self
         """
 
         if not short_name:
@@ -417,13 +424,13 @@ class GranuleCollectionBaseQuery(Query):
         self.params['short_name'] = short_name
         return self
 
-    def version(self, version):
+    def version(self, version: str) -> Self:
         """
         Filter by version. Note that CMR defines this as a string. For example,
         MODIS version 6 products must be searched for with "006".
 
         :param version: version string
-        :returns: Query instance
+        :returns: self
         """
 
         if not version:
@@ -432,45 +439,42 @@ class GranuleCollectionBaseQuery(Query):
         self.params['version'] = version
         return self
 
-    def point(self, lon, lat):
+    def point(self, lon: FloatLike, lat: FloatLike) -> Self:
         """
         Filter by granules that include a geographic point.
 
         :param lon: longitude of geographic point
         :param lat: latitude of geographic point
-        :returns: Query instance
+        :returns: self
         """
 
-        if not lat or not lon:
-            return self
-
         # coordinates must be a float
         lon = float(lon)
         lat = float(lat)
 
-        self.params['point'] = "{},{}".format(lon, lat)
+        self.params['point'] = f"{lon},{lat}"
 
         return self
 
-    def circle(self, lon: float, lat: float, dist: int):
+    def circle(self, lon: FloatLike, lat: FloatLike, dist: FloatLike) -> Self:
         """Filter by granules within the circle around lat/lon
 
         :param lon: longitude of geographic point
         :param lat: latitude of geographic point
         :param dist: distance in meters around waypoint (lat,lon)
-        :returns: Query instance
+        :returns: self
         """
         self.params['circle'] = f"{lon},{lat},{dist}"
 
         return self
 
-    def polygon(self, coordinates):
+    def polygon(self, coordinates: Sequence[PointLike]) -> Self:
         """
         Filter by granules that overlap a polygonal area. Must be used in combination with a
         collection filtering parameter such as short_name or entry_title.
 
         :param coordinates: list of (lon, lat) tuples
-        :returns: Query instance
+        :returns: self
         """
 
         if not coordinates:
@@ -480,11 +484,15 @@ class GranuleCollectionBaseQuery(Query):
         try:
             iter(coordinates)
         except TypeError:
-            raise ValueError("A line must be an iterable of coordinate tuples. Ex: [(90,90), (91, 90), ...]")
+            raise TypeError(
+                f"A line must be an iterable of coordinate tuples. Ex: [(90,90), (91, 90), ...]; got {type(coordinates)}."
+            ) from None
 
         # polygon requires at least 4 pairs of coordinates
         if len(coordinates) < 4:
-            raise ValueError("A polygon requires at least 4 pairs of coordinates.")
+            raise ValueError(
+                f"A polygon requires at least 4 pairs of coordinates; got {len(coordinates)}."
+            )
 
         # convert to floats
         as_floats = []
@@ -493,7 +501,9 @@ class GranuleCollectionBaseQuery(Query):
 
         # last point must match first point to complete polygon
         if as_floats[0] != as_floats[-2] or as_floats[1] != as_floats[-1]:
-            raise ValueError("Coordinates of the last pair must match the first pair.")
+            raise ValueError(
+                f"Coordinates of the last pair must match the first pair: {coordinates[0]} != {coordinates[-1]}"
+            )
 
         # convert to strings
         as_strs = [str(val) for val in as_floats]
@@ -502,7 +512,13 @@ class GranuleCollectionBaseQuery(Query):
 
         return self
 
-    def bounding_box(self, lower_left_lon, lower_left_lat, upper_right_lon, upper_right_lat):
+    def bounding_box(
+        self,
+        lower_left_lon: FloatLike,
+        lower_left_lat: FloatLike,
+        upper_right_lon: FloatLike,
+        upper_right_lat: FloatLike,
+    ) -> Self:
         """
         Filter by granules that overlap a bounding box. Must be used in combination with
         a collection filtering parameter such as short_name or entry_title.
@@ -511,25 +527,22 @@ class GranuleCollectionBaseQuery(Query):
         :param lower_left_lat: lower left latitude of the box
         :param upper_right_lon: upper right longitude of the box
         :param upper_right_lat: upper right latitude of the box
-        :returns: Query instance
+        :returns: self
         """
 
-        self.params["bounding_box"] = "{},{},{},{}".format(
-            float(lower_left_lon),
-            float(lower_left_lat),
-            float(upper_right_lon),
-            float(upper_right_lat)
+        self.params["bounding_box"] = (
+            f"{float(lower_left_lon)},{float(lower_left_lat)},{float(upper_right_lon)},{float(upper_right_lat)}"
         )
 
         return self
 
-    def line(self, coordinates):
+    def line(self, coordinates: Sequence[PointLike]) -> Self:
         """
         Filter by granules that overlap a series of connected points. Must be used in combination
         with a collection filtering parameter such as short_name or entry_title.
 
         :param coordinates: a list of (lon, lat) tuples
-        :returns: Query instance
+        :returns: self
         """
 
         if not coordinates:
@@ -539,11 +552,15 @@ class GranuleCollectionBaseQuery(Query):
         try:
             iter(coordinates)
         except TypeError:
-            raise ValueError("A line must be an iterable of coordinate tuples. Ex: [(90,90), (91, 90), ...]")
+            raise TypeError(
+                f"A line must be an iterable of coordinate tuples. Ex: [(90,90), (91, 90), ...]; got {type(coordinates)}."
+            ) from None
 
         # need at least 2 pairs of coordinates
         if len(coordinates) < 2:
-            raise ValueError("A line requires at least 2 pairs of coordinates.")
+            raise ValueError(
+                f"A line requires at least 2 pairs of coordinates; got {len(coordinates)}."
+            )
 
         # make sure they're all floats
         as_floats = []
@@ -557,13 +574,13 @@ class GranuleCollectionBaseQuery(Query):
 
         return self
 
-    def downloadable(self, downloadable=True):
+    def downloadable(self, downloadable: bool = True) -> Self:
         """
         Only match granules that are available for download. The opposite of this
         method is online_only().
 
         :param downloadable: True to require granules be downloadable
-        :returns: Query instance
+        :returns: self
         """
 
         if not isinstance(downloadable, bool):
@@ -577,12 +594,12 @@ class GranuleCollectionBaseQuery(Query):
 
         return self
 
-    def entry_title(self, entry_title):
+    def entry_title(self, entry_title: str) -> Self:
         """
         Filter by the collection entry title.
 
         :param entry_title: Entry title of the collection
-        :returns: Query instance
+        :returns: self
         """
 
         entry_title = quote(entry_title)
@@ -597,53 +614,56 @@ class GranuleQuery(GranuleCollectionBaseQuery):
     Class for querying granules from the CMR.
     """
 
-    def __init__(self, mode=CMR_OPS):
+    def __init__(self, mode: str = CMR_OPS):
         Query.__init__(self, "granules", mode)
-        self.concept_id_chars = ['G', 'C']
+        self.concept_id_chars = {"G", "C"}
 
-    def orbit_number(self, orbit1, orbit2=None):
-        """"
+    def orbit_number(
+        self, orbit1: FloatLike, orbit2: Optional[FloatLike] = None
+    ) -> Self:
+        """ "
         Filter by the orbit number the granule was acquired during. Either a single
         orbit can be targeted or a range of orbits.
 
         :param orbit1: orbit to target (lower limit of range when orbit2 is provided)
         :param orbit2: upper limit of range
-        :returns: Query instance
+        :returns: self
         """
 
         if orbit2:
-            self.params['orbit_number'] = quote('{},{}'.format(str(orbit1), str(orbit2)))
+            self.params['orbit_number'] = quote(f'{str(orbit1)},{str(orbit2)}')
         else:
             self.params['orbit_number'] = orbit1
 
         return self
 
-    def day_night_flag(self, day_night_flag):
+    def day_night_flag(self, day_night_flag: DayNightFlag) -> Self:
         """
         Filter by period of the day the granule was collected during.
 
         :param day_night_flag: "day", "night", or "unspecified"
-        :returns: Query instance
+        :returns: self
         """
 
         if not isinstance(day_night_flag, str):
             raise TypeError("day_night_flag must be of type str.")
 
-        day_night_flag = day_night_flag.lower()
+        if day_night_flag.lower() not in ["day", "night", "unspecified"]:
+            raise ValueError(
+                "day_night_flag must be 'day', 'night', or 'unspecified': "
+                f"{day_night_flag!r}."
+            )
 
-        if day_night_flag not in ['day', 'night', 'unspecified']:
-            raise ValueError("day_night_flag must be day, night or unspecified.")
-
-        self.params['day_night_flag'] = day_night_flag
+        self.params["day_night_flag"] = day_night_flag.lower()
         return self
 
-    def cloud_cover(self, min_cover=0, max_cover=100):
+    def cloud_cover(self, min_cover: FloatLike = 0, max_cover: FloatLike = 100) -> Self:
         """
         Filter by the percentage of cloud cover present in the granule.
 
         :param min_cover: minimum percentage of cloud cover
         :param max_cover: maximum percentage of cloud cover
-        :returns: Query instance
+        :returns: self
         """
 
         if not min_cover and not max_cover:
@@ -655,19 +675,27 @@ class GranuleQuery(GranuleCollectionBaseQuery):
                 maxiumum = float(max_cover)
 
                 if minimum > maxiumum:
-                    raise ValueError("Please ensure min_cloud_cover is lower than max cloud cover")
+                    raise ValueError(
+                        "Please ensure min cloud cover is less than max cloud cover"
+                    )
             except ValueError:
-                raise ValueError("Please ensure min_cover and max_cover are both floats")
+                raise ValueError(
+                    "Please ensure min_cover and max_cover are both floats"
+                ) from None
+            except TypeError:
+                raise TypeError(
+                    "Please ensure min_cover and max_cover are both convertible to floats"
+                ) from None
 
-        self.params['cloud_cover'] = "{},{}".format(min_cover, max_cover)
+        self.params['cloud_cover'] = f"{min_cover},{max_cover}"
         return self
 
-    def instrument(self, instrument=""):
+    def instrument(self, instrument: str) -> Self:
         """
         Filter by the instrument associated with the granule.
 
         :param instrument: name of the instrument
-        :returns: Query instance
+        :returns: self
         """
 
         if not instrument:
@@ -676,12 +704,12 @@ class GranuleQuery(GranuleCollectionBaseQuery):
         self.params['instrument'] = instrument
         return self
 
-    def platform(self, platform=""):
+    def platform(self, platform: str) -> Self:
         """
         Filter by the satellite platform the granule came from.
 
         :param platform: name of the satellite
-        :returns: Query instance
+        :returns: self
         """
 
         if not platform:
@@ -690,59 +718,62 @@ class GranuleQuery(GranuleCollectionBaseQuery):
         self.params['platform'] = platform
         return self
 
-
-    def sort_key(self, sort_key=""):
+    def sort_key(self, sort_key: str) -> Self:
         """
         See
         https://cmr.earthdata.nasa.gov/search/site/docs/search/api.html#sorting-granule-results
         for valid granule sort_keys
 
-        Filter some defined sort_key; 
+        Filter some defined sort_key;
         use negative (-) for start_date and end_date to sort by ascending
 
         :param sort_key: name of the sort key
-        :returns: Query instance
+        :returns: self
         """
 
-        valid_sort_keys = [
-        'campaign',
-        'entry_title',
-        'dataset_id',
-        'data_size',
-        'end_date',
-        '-end_date'
-        'granule_ur',
-        'producer_granule_id'
-        'project',
-        'provider',
-        'readable_granule_name',
-        'short_name',
-        '-start_date',
-        'start_date',
-        'version',
-        'platform',
-        'instrument',
-        'sensor',
-        'day_night_flag',
-        'online_only',
-        'browsable',
-        'browse_only',
-        'cloud_cover',
-        'revision_date']
-        # also covers if empty string
-        if sort_key not in valid_sort_keys:
-            raise ValueError("Please provide a valid sort_key for granules query see https://cmr.earthdata.nasa.gov/search/site/docs/search/api.html#sorting-granule-results for valid sort_keys")
+        valid_sort_keys = {
+            'campaign',
+            'entry_title',
+            'dataset_id',
+            'data_size',
+            'end_date',
+            'granule_ur',
+            'producer_granule_id',
+            'project',
+            'provider',
+            'readable_granule_name',
+            'short_name',
+            'start_date',
+            'version',
+            'platform',
+            'instrument',
+            'sensor',
+            'day_night_flag',
+            'online_only',
+            'browsable',
+            'browse_only',
+            'cloud_cover',
+            'revision_date',
+        }
+
+        # also covers if empty string and allows for '-' prefix (for descending order)
+        if not isinstance(sort_key, str) or sort_key.lstrip('-') not in valid_sort_keys:
+            raise ValueError(
+                "Please provide a valid sort key for granules query.  See"
+                " https://cmr.earthdata.nasa.gov/search/site/docs/search/api.html#sorting-granule-results"
+                " for valid sort keys."
+            )
 
         self.params['sort_key'] = sort_key
         return self
 
-    def granule_ur(self, granule_ur=""):
+    def granule_ur(self, granule_ur: str) -> Self:
         """
         Filter by the granules unique ID. Note this will result in at most one granule
         being returned.
 
         :param granule_ur: UUID of a granule
-        :returns: Query instance
+        :returns: self
         """
 
         if not granule_ur:
@@ -750,8 +781,10 @@ class GranuleQuery(GranuleCollectionBaseQuery):
 
         self.params['granule_ur'] = granule_ur
         return self
-    
-    def readable_granule_name(self, readable_granule_name=""):
+
+    def readable_granule_name(
+        self, readable_granule_name: Union[str, Sequence[str]]
+    ) -> Self:
         """
         Filter by the readable granule name (producer_granule_id if present, otherwise producer_granule_id).
 
@@ -761,18 +794,19 @@ class GranuleQuery(GranuleCollectionBaseQuery):
         question mark (?) will match exactly one character.
 
         :param readable_granule_name: granule name or substring
-        :returns: Query instance
+        :returns: self
         """
 
         if isinstance(readable_granule_name, str):
             readable_granule_name = [readable_granule_name]
-        
+
         self.params["readable_granule_name"] = readable_granule_name
         self.options["readable_granule_name"] = {"pattern": True}
 
         return self
 
-    def _valid_state(self):
+    @override
+    def _valid_state(self) -> bool:
 
         # spatial params must be paired with a collection limiting parameter
         spatial_keys = ["point", "polygon", "bounding_box", "line"]
@@ -791,19 +825,19 @@ class CollectionQuery(GranuleCollectionBaseQuery):
     Class for querying collections from the CMR.
     """
 
-    def __init__(self, mode=CMR_OPS):
+    def __init__(self, mode: str = CMR_OPS):
         Query.__init__(self, "collections", mode)
-        self.concept_id_chars = ['C']
+        self.concept_id_chars = {"C"}
         self._valid_formats_regex.extend([
             "dif", "dif10", "opendata", "umm_json", "umm_json_v[0-9]_[0-9]"
         ])
 
-    def archive_center(self, center):
+    def archive_center(self, center: str) -> Self:
         """
         Filter by the archive center that maintains the collection.
 
         :param archive_center: name of center as a string
-        :returns: Query instance
+        :returns: self
         """
 
         if center:
@@ -811,14 +845,14 @@ class CollectionQuery(GranuleCollectionBaseQuery):
 
         return self
 
-    def keyword(self, text):
+    def keyword(self, text: str) -> Self:
         """
         Case insentive and wildcard (*) search through over two dozen fields in
         a CMR collection record. This allows for searching against fields like
         summary and science keywords.
 
         :param text: text to search for
-        :returns: Query instance
+        :returns: self
         """
 
         if text:
@@ -826,12 +860,12 @@ class CollectionQuery(GranuleCollectionBaseQuery):
 
         return self
 
-    def native_id(self, native_ids):
+    def native_id(self, native_ids: Union[str, Sequence[str]]) -> Self:
         """
         Filter by native id.
 
         :param native_id: native id for collection
-        :returns: Query instance
+        :returns: self
         """
 
         if isinstance(native_ids, str):
@@ -841,14 +875,14 @@ class CollectionQuery(GranuleCollectionBaseQuery):
 
         return self
 
-    def tool_concept_id(self, IDs):
+    def tool_concept_id(self, IDs: Union[str, Sequence[str]]) -> Self:
         """
         Filter collections associated with tool concept ID (ex: TL2092786348-POCLOUD)
 
         Collections are associated with this tool ID.
 
         :param IDs: tool concept ID(s) to search by. Can be provided as a string or list of strings.
-        :returns: Query instance
+        :returns: self
         """
 
         if isinstance(IDs, str):
@@ -857,20 +891,20 @@ class CollectionQuery(GranuleCollectionBaseQuery):
         # verify we provided with tool concept IDs
         for ID in IDs:
             if ID.strip()[0] != "T":
-                raise ValueError("Only tool concept ID's can be provided (begin with 'T'): {}".format(ID))
+                raise ValueError(f"Only tool concept ID's can be provided (begin with 'T'): {ID}")
 
         self.params["tool_concept_id"] = IDs
 
         return self
 
-    def service_concept_id(self, IDs):
+    def service_concept_id(self, IDs: Union[str, Sequence[str]]) -> Self:
         """
         Filter collections associated with service ID (ex: S1962070864-POCLOUD)
 
         Collections are associated with this service ID.
 
         :param IDs: service concept ID(s) to search by. Can be provided as a string or list of strings.
-        :returns: Query instance
+        :returns: self
         """
 
         if isinstance(IDs, str):
@@ -879,13 +913,16 @@ class CollectionQuery(GranuleCollectionBaseQuery):
         # verify we provided with service concept IDs
         for ID in IDs:
             if ID.strip()[0] != "S":
-                raise ValueError("Only service concept ID's can be provided (begin with 'S'): {}".format(ID))
+                raise ValueError(
+                    f"Only service concept IDs can be provided (begin with 'S'): {ID}"
+                )
 
         self.params["service_concept_id"] = IDs
 
         return self
 
-    def _valid_state(self):
+    @override
+    def _valid_state(self) -> bool:
         return True
 
 
@@ -894,7 +931,7 @@ class ToolServiceVariableBaseQuery(Query):
     Base class for Tool and Service CMR queries.
     """
 
-    def get(self, limit=2000):
+    def get(self, limit: int = 2000) -> Sequence[Any]:
         """
         Get all results up to some limit, even if spanning multiple pages.
 
@@ -905,16 +942,14 @@ class ToolServiceVariableBaseQuery(Query):
         page_size = min(limit, 2000)
         url = self._build_url()
 
-        results = []
+        results: List[Any] = []
         page = 1
         while len(results) < limit:
 
-            response = get(url, params={'page_size': page_size, 'page_num': page})
-
-            try:
-                response.raise_for_status()
-            except exceptions.HTTPError as ex:
-                raise RuntimeError(ex.response.text)
+            response = requests.get(
+                url, params={"page_size": page_size, "page_num": page}
+            )
+            response.raise_for_status()
 
             if self._format == "json":
                 latest = response.json()['items']
@@ -929,12 +964,12 @@ class ToolServiceVariableBaseQuery(Query):
 
         return results
 
-    def native_id(self, native_ids):
+    def native_id(self, native_ids: Union[str, Sequence[str]]) -> Self:
         """
         Filter by native id.
 
         :param native_id: native id for tool or service
-        :returns: Query instance
+        :returns: self
         """
 
         if isinstance(native_ids, str):
@@ -944,12 +979,12 @@ class ToolServiceVariableBaseQuery(Query):
 
         return self
 
-    def name(self, name):
+    def name(self, name: str) -> Self:
         """
         Filter by name.
 
         :param name: name of service or tool.
-        :returns: Query instance
+        :returns: self
         """
 
         if not name:
@@ -964,14 +999,15 @@ class ToolQuery(ToolServiceVariableBaseQuery):
     Class for querying tools from the CMR.
     """
 
-    def __init__(self, mode=CMR_OPS):
+    def __init__(self, mode: str = CMR_OPS):
         Query.__init__(self, "tools", mode)
-        self.concept_id_chars = ['T']
+        self.concept_id_chars = {"T"}
         self._valid_formats_regex.extend([
             "dif", "dif10", "opendata", "umm_json", "umm_json_v[0-9]_[0-9]"
         ])
 
-    def _valid_state(self):
+    @override
+    def _valid_state(self) -> bool:
         return True
 
 
@@ -980,24 +1016,27 @@ class ServiceQuery(ToolServiceVariableBaseQuery):
     Class for querying services from the CMR.
     """
 
-    def __init__(self, mode=CMR_OPS):
+    def __init__(self, mode: str = CMR_OPS):
         Query.__init__(self, "services", mode)
-        self.concept_id_chars = ['S']
+        self.concept_id_chars = {"S"}
         self._valid_formats_regex.extend([
             "dif", "dif10", "opendata", "umm_json", "umm_json_v[0-9]_[0-9]"
         ])
 
-    def _valid_state(self):
+    @override
+    def _valid_state(self) -> bool:
         return True
 
 
 class VariableQuery(ToolServiceVariableBaseQuery):
-    def __init__(self, mode=CMR_OPS):
+
+    def __init__(self, mode: str = CMR_OPS):
         Query.__init__(self, "variables", mode)
-        self.concept_id_chars = ['V']
+        self.concept_id_chars = {"V"}
         self._valid_formats_regex.extend([
             "dif", "dif10", "opendata", "umm_json", "umm_json_v[0-9]_[0-9]"
         ])
 
-    def _valid_state(self):
+    @override
+    def _valid_state(self) -> bool:
         return True
